@@ -21,7 +21,7 @@
 #include "aubio_priv.h"
 #include "cvec.h"
 
-cvec_t * new_cvec( uint_t length) {
+cvec_t * new_cvec(uint_t length) {
   cvec_t * s;
   if ((sint_t)length <= 0) {
     return NULL;
@@ -55,17 +55,17 @@ smpl_t cvec_phas_get_sample (cvec_t *s, uint_t position) {
   return s->phas[position];
 }
 
-smpl_t * cvec_norm_get_data (cvec_t *s) {
+smpl_t * cvec_norm_get_data (const cvec_t *s) {
   return s->norm;
 }
 
-smpl_t * cvec_phas_get_data (cvec_t *s) {
+smpl_t * cvec_phas_get_data (const cvec_t *s) {
   return s->phas;
 }
 
 /* helper functions */
 
-void cvec_print(cvec_t *s) {
+void cvec_print(const cvec_t *s) {
   uint_t j;
   AUBIO_MSG("norm: ");
   for (j=0; j< s->length; j++) {
@@ -79,22 +79,22 @@ void cvec_print(cvec_t *s) {
   AUBIO_MSG("\n");
 }
 
-void cvec_copy(cvec_t *s, cvec_t *t) {
+void cvec_copy(const cvec_t *s, cvec_t *t) {
   if (s->length != t->length) {
     AUBIO_ERR("trying to copy %d elements to %d elements \n",
         s->length, t->length);
     return;
   }
-#if HAVE_MEMCPY_HACKS
+#ifdef HAVE_MEMCPY_HACKS
   memcpy(t->norm, s->norm, t->length * sizeof(smpl_t));
   memcpy(t->phas, s->phas, t->length * sizeof(smpl_t));
-#else
+#else /* HAVE_MEMCPY_HACKS */
   uint_t j;
   for (j=0; j< t->length; j++) {
     t->norm[j] = s->norm[j];
     t->phas[j] = s->phas[j];
   }
-#endif
+#endif /* HAVE_MEMCPY_HACKS */
 }
 
 void cvec_norm_set_all (cvec_t *s, smpl_t val) {
@@ -105,11 +105,11 @@ void cvec_norm_set_all (cvec_t *s, smpl_t val) {
 }
 
 void cvec_norm_zeros(cvec_t *s) {
-#if HAVE_MEMCPY_HACKS
+#ifdef HAVE_MEMCPY_HACKS
   memset(s->norm, 0, s->length * sizeof(smpl_t));
-#else
+#else /* HAVE_MEMCPY_HACKS */
   cvec_norm_set_all (s, 0.);
-#endif
+#endif /* HAVE_MEMCPY_HACKS */
 }
 
 void cvec_norm_ones(cvec_t *s) {
@@ -124,7 +124,7 @@ void cvec_phas_set_all (cvec_t *s, smpl_t val) {
 }
 
 void cvec_phas_zeros(cvec_t *s) {
-#if HAVE_MEMCPY_HACKS
+#ifdef HAVE_MEMCPY_HACKS
   memset(s->phas, 0, s->length * sizeof(smpl_t));
 #else
   cvec_phas_set_all (s, 0.);
