@@ -2,6 +2,7 @@
 
 int main (void)
 {
+  int return_code = 0;
   uint_t i, n_iters = 100; // number of iterations
   uint_t win_s = 500; // window size
   fvec_t * in = new_fvec (win_s); // input buffer
@@ -10,6 +11,11 @@ int main (void)
   // create fft object
   aubio_fft_t * fft = new_aubio_fft(win_s);
 
+  if (!fft) {
+    return_code = 1;
+    goto beach;
+  }
+
   // fill input with some data
   in->data[0] = 1;
   in->data[1] = 2;
@@ -33,9 +39,10 @@ int main (void)
   // cleam up
   //fvec_print(out);
   del_aubio_fft(fft);
+beach:
   del_fvec(in);
   del_cvec(fftgrain);
   del_fvec(out);
   aubio_cleanup();
-  return 0;
+  return return_code;
 }
